package ue121_Figuren;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import java.awt.Color;

/****************************************************************************************************
 * 
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Figur
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
//	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
//	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";
	private static final String errNoException = "Hier darf keine Exception ausgelöst werden!";
	
	// -------------------------------------------------------------------------------------
	@BeforeEach
	void setUp() throws Exception
	{
		  // Hier nicht benötigt
	}
	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(10)
	void testKreis()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Kreis(10,15, null, true, 25);
		}, errNewNullPointer);

		assertThrows(IllegalArgumentException.class, () -> {
			new Kreis(10,15, Color.red, true, 0);
		}, errArgIllegal + "Radius > 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Kreis(10,15, Color.red, true, 25);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Kreis kreis = new Kreis(10,15, Color.red, true, 25);
		
		assertEquals(10, 		kreis.getX(),	  errReturnValue + "getX");
		assertEquals(15,  		kreis.getY(),	  errReturnValue + "getY");
		assertEquals(Color.red, kreis.getFarbe(), errReturnValue + "getFarbe");
		assertEquals(25,  		kreis.getRadius(),errReturnValue + "getRadius");		

		// --------------------------------------------------------------------------------
		kreis.setX(25);
		assertEquals(25,	kreis.getX(),	errReturnValue + "getX");
		kreis.setY(35);
		assertEquals(35,	kreis.getY(),	errReturnValue + "getY");

		// ------------------------------------------------------------------------------
		assertThrows(NullPointerException.class, () -> {
			kreis.setFarbe(null);
		}, errArgIllegal + "setFarbe darf nicht null sein!");

		kreis.setFarbe(Color.blue);
		assertEquals(Color.blue, kreis.getFarbe(), errReturnValue + "getFarbe");
		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			kreis.setRadius(0);
		}, errArgIllegal + "Radius > 0");

		kreis.setRadius(125);
		assertEquals(125,	kreis.getRadius(),	errReturnValue + "getRadius");

		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			kreis.paint();
			kreis.toString();
		}, errNoException );
	}

	// -------------------------------------------------------------------------------------
	@Test
	@Order(20)
	void testQuadrat()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Quadrat(10,15, null, true, 25);
		}, errNewNullPointer);

		assertThrows(IllegalArgumentException.class, () -> {
			new Quadrat(10,15, Color.red, true, 0);
		}, errArgIllegal + "Seitenlänge > 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Quadrat(10,15, Color.red, true, 25);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Quadrat quadrat = new Quadrat(10,15, Color.red, true, 25);
		
		assertEquals(10, 		quadrat.getX(),	  errReturnValue + "getX");
		assertEquals(15,  		quadrat.getY(),	  errReturnValue + "getY");
		assertEquals(Color.red, quadrat.getFarbe(), errReturnValue + "getFarbe");
		assertEquals(25,  		quadrat.getSeitenlaenge(),errReturnValue + "getSeitenlaenge");		

		// --------------------------------------------------------------------------------
		quadrat.setX(25);
		assertEquals(25,	quadrat.getX(),	errReturnValue + "getX");
		quadrat.setY(35);
		assertEquals(35,	quadrat.getY(),	errReturnValue + "getY");

		// ------------------------------------------------------------------------------
		assertThrows(NullPointerException.class, () -> {
			quadrat.setFarbe(null);
		}, errArgIllegal + "setFarbe darf nicht null sein!");

		quadrat.setFarbe(Color.blue);
		assertEquals(Color.blue, quadrat.getFarbe(), errReturnValue + "getFarbe");
		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			quadrat.setSeitenlaenge(0);
		}, errArgIllegal + "Länge > 0");

		quadrat.setSeitenlaenge(125);
		assertEquals(125,	quadrat.getSeitenlaenge(),	errReturnValue + "getSeitenlaenge");

		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			quadrat.paint();
			quadrat.toString();
		}, errNoException );
	}

	// -------------------------------------------------------------------------------------
	@Test
	@Order(20)
	void testRechteck()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Rechteck(10,15, null, true, 25, 45);
		}, errNewNullPointer);

		assertThrows(IllegalArgumentException.class, () -> {
			new Rechteck(10,15, Color.red, true, 0, 45);
		}, errArgIllegal + "Länge > 0");

		assertThrows(IllegalArgumentException.class, () -> {
			new Rechteck(10,15, Color.red, true, 25, 0);
		}, errArgIllegal + "Breite > 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Rechteck(10,15, Color.red, true, 25, 45);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Rechteck rechteck = new Rechteck(10,15, Color.red, true, 25, 45);
		
		assertEquals(10, 		rechteck.getX(),	  errReturnValue + "getX");
		assertEquals(15,  		rechteck.getY(),	  errReturnValue + "getY");
		assertEquals(Color.red, rechteck.getFarbe(), errReturnValue + "getFarbe");
		assertEquals(25,  		rechteck.getLaenge(),errReturnValue + "getLaenge");		
		assertEquals(45,  		rechteck.getBreite(),errReturnValue + "getBreite");		

		// --------------------------------------------------------------------------------
		rechteck.setX(25);
		assertEquals(25,	rechteck.getX(),	errReturnValue + "getX");
		rechteck.setY(35);
		assertEquals(35,	rechteck.getY(),	errReturnValue + "getY");

		// ------------------------------------------------------------------------------
		assertThrows(NullPointerException.class, () -> {
			rechteck.setFarbe(null);
		}, errArgIllegal + "setFarbe darf nicht null sein!");

		rechteck.setFarbe(Color.blue);
		assertEquals(Color.blue, rechteck.getFarbe(), errReturnValue + "getFarbe");
		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			rechteck.setLaenge(0);
		}, errArgIllegal + "Länge > 0");

		rechteck.setLaenge(125);
		assertEquals(125,	rechteck.getLaenge(),	errReturnValue + "getLaenge");

		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			rechteck.setBreite(0);
		}, errArgIllegal + "Breite > 0");

		rechteck.setBreite(125);
		assertEquals(125,	rechteck.getBreite(),	errReturnValue + "getBreite");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			rechteck.paint();
			rechteck.toString();
		}, errNoException );
	}

	// -------- Ende der Klasse 
}
