package ue211_Fuhrpark;
import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;

/***************************************************************************************************
 * 
 * JUnit Test für Fahrzeuge
 *
 * @author Sven Marceta
 * @version 1.0
 *
 ***************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Fahrzeug
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
//	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";

	@Test
	void testAuto()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Auto(null, "Auto", 'S', 'C');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Auto("", "Auto", 'S', 'C');
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Auto("FIN001", null, 'D', 'C');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Auto("FIN001", "", 'D', 'C');
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Auto("FIN001", "Auto", 'X', 'C');
		}, errArgIllegal + "Treibstoff");
		assertThrows(IllegalArgumentException.class, () -> {
			new Auto("FIN001", "Auto", 'D', 'X');
		}, errArgIllegal + "Bauart");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Auto("FIN001", "Auto", 'S', 'C');
		}, errNewNoError);
	
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Auto neu = new Auto("FIN001", "Auto", 'S', 'C');
		
		assertEquals("FIN001", neu.getFin(),        errReturnValue + "getFin");
		assertEquals("Auto",   neu.getMarke(),      errReturnValue + "getMarke");
		assertEquals('S',      neu.getTreibstoff(), errReturnValue + "getTreibstoff");
		assertEquals('C',      neu.getBauart(),     errReturnValue + "getBauart");
		
		// --------------------------------------------------------------------------------
		assertEquals(0, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		
		assertEquals(8, neu.beschleunigen(4, 2), errReturnValue + "bechleunigen");
		assertEquals(8, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		assertEquals(4, neu.bremsen(2, 2),       errReturnValue + "bremsen");
		assertEquals(4, neu.getvAktuell(),       errReturnValue + "getvAktuell");
	}

	@Test
	void testCabrio()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Cabrio(null, "Cabrio", 'S', 'C');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Cabrio("", "Cabrio", 'S', 'C');
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Cabrio("FIN001", null, 'D', 'C');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Cabrio("FIN001", "", 'D', 'C');
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Cabrio("FIN001", "Cabrio", 'X', 'C');
		}, errArgIllegal + "Treibstoff!");
		assertThrows(IllegalArgumentException.class, () -> {
			new Cabrio("FIN001", "Cabrio", 'S', 'X');
		}, errArgIllegal + "Bauart!");
		
		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Cabrio("FIN001", "Cabrio", 'S', 'C');
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Cabrio neu = new Cabrio("FIN001", "Cabrio", 'S', 'C');
		
		assertEquals("FIN001", neu.getFin(),        errReturnValue + "getFin");
		assertEquals("Cabrio", neu.getMarke(),    errReturnValue + "getMarke");
		assertEquals('S',      neu.getTreibstoff(), errReturnValue + "getTreibstoff");
		assertEquals('C',      neu.getBauart(),     errReturnValue + "getBauart");

		neu.setVerdeckoffen(true);
		
		// --------------------------------------------------------------------------------
		assertEquals(0, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		
		assertEquals(8, neu.beschleunigen(4, 2), errReturnValue + "bechleunigen");
		assertEquals(8, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		assertEquals(4, neu.bremsen(2, 2),       errReturnValue + "bremsen");
		assertEquals(4, neu.getvAktuell(),       errReturnValue + "getvAktuell");
	}
	
	@Test
	void testLKW()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new LKW(null, "LKW", 'S', 15000);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new LKW("", "LKW", 'S', 15000);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new LKW("FIN001", null, 'D', 15000);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new LKW("FIN001", "", 'D', 15000);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new LKW("FIN001", "LKW", 'X', 0);
		}, errArgIllegal + "Treibstoff!");
		assertThrows(IllegalArgumentException.class, () -> {
			new LKW("FIN001", "LKW", 'S', 50001);
		}, errArgIllegal + "max.Ladung!");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new LKW("FIN001", "LKW", 'S', 15000);
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		LKW neu = new LKW("FIN001", "LKW", 'S', 15000);
		
		assertEquals("FIN001", neu.getFin(),        errReturnValue + "getFin");
		assertEquals("LKW",    neu.getMarke(),       errReturnValue + "getMarke");
		assertEquals('S',      neu.getTreibstoff(), errReturnValue + "getTreibstoff");
		assertEquals(15000,    neu.getMaxGewicht(), errReturnValue + "getMaxGewicht");

		assertTrue(neu.beladen(2000),  errReturnValue + "beladen");
		assertTrue(neu.beladen(8000),  errReturnValue + "beladen");
		assertTrue(neu.beladen(5000),  errReturnValue + "beladen");
		assertFalse(neu.beladen(1),    errReturnValue + "beladen");
		
		// --------------------------------------------------------------------------------
		assertEquals(0, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		
		assertEquals(8, neu.beschleunigen(4, 2), errReturnValue + "bechleunigen");
		assertEquals(8, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		assertEquals(4, neu.bremsen(2, 2),       errReturnValue + "bremsen");
		assertEquals(4, neu.getvAktuell(),       errReturnValue + "getvAktuell");
	}

	@Test
	void testMotorrad()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Motorrad(null, "Motorrad", 'S', 150);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Motorrad("", "Motorrad", 'S', 150);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Motorrad("FIN001", null, 'D', 150);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Motorrad("FIN001", "", 'D', 150);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Motorrad("FIN001", "Motorrad", 'X', 150);
		}, errArgIllegal + "Treibstoff!");
		assertThrows(IllegalArgumentException.class, () -> {
			new Motorrad("FIN001", "Motorrad", 'S', 250);
		}, errArgIllegal + "Leistung!");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Motorrad("FIN001", "Motorrad", 'S', 150);
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Motorrad neu = new Motorrad("FIN001", "Motorrad", 'S', 150);
		
		assertEquals("FIN001",  neu.getFin(),        errReturnValue + "getFin");
		assertEquals("Motorrad",neu.getMarke(),      errReturnValue + "getMarke");
		assertEquals('S',       neu.getTreibstoff(), errReturnValue + "getTreibstoff");
		assertEquals(150,       neu.getLeistung(),   errReturnValue + "getLeistung");

		// --------------------------------------------------------------------------------
		assertEquals(0, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		
		assertEquals(8, neu.beschleunigen(4, 2), errReturnValue + "bechleunigen");
		assertEquals(8, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		assertEquals(4, neu.bremsen(2, 2),       errReturnValue + "bremsen");
		assertEquals(4, neu.getvAktuell(),       errReturnValue + "getvAktuell");
	}

	@Test
	void testMoped()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Moped(null, "Moped", 'S');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Moped("", "Moped", 'S');
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Moped("FIN001", null, 'D');
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Moped("FIN001", "", 'D');
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Moped("FIN001", "Moped", 'X');
		}, errArgIllegal + "Treibstoff!");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Moped("FIN001", "Moped", 'S');
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Moped neu = new Moped("FIN001", "Moped", 'S');
		
		assertEquals("FIN001", neu.getFin(),          errReturnValue + "getFIN");
		assertEquals("Moped",   neu.getMarke(),       errReturnValue + "getMarke");
		assertEquals('S',       neu.getTreibstoff(),  errReturnValue + "getTreibstoff");

		// --------------------------------------------------------------------------------
		assertEquals(0, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		
		assertEquals(8, neu.beschleunigen(4, 2), errReturnValue + "bechleunigen");
		assertEquals(8, neu.getvAktuell(),       errReturnValue + "getvAktuell");
		assertEquals(4, neu.bremsen(2, 2),       errReturnValue + "bremsen");
		assertEquals(4, neu.getvAktuell(),       errReturnValue + "getvAktuell");
	}
}
