package ue221_Figurenliste;
import static org.junit.jupiter.api.Assertions.*;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;

import sew.G2D;

/****************************************************************************************************
 * 
 * JUnit Test für Klasse Figurenliste.
 *
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_FigurenListe
{
//	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
//	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
//	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
//	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";
	private static final String errNoException = "Hier darf keine Exception ausgelöst werden!";

	// -----------------------------------------------------------------
	// Variablen für Testumgebung
	FigurenListe     testListe;

	Kreis      kreis1, kreis2;
	Quadrat    quad1, quad2;
	Rechteck   recht1, recht2;

	// -----------------------------------------------------------------------------------
	@BeforeEach
	void setUp() throws Exception
	{
		testListe = new FigurenListe();

		// Die einzelnen Klassen wurden bereits in einem anderen JUnit-Test getestet
		kreis1   = new Kreis(10,10, Color.red, true, 15);
		kreis2   = new Kreis(20,20, Color.blue, true, 25);
		recht1   = new Rechteck(30,30, Color.red, true, 15,25);
		recht2   = new Rechteck(40,40, Color.blue, true, 25,35);
		quad1    = new Quadrat(50,50, Color.red, true, 15);
		quad2    = new Quadrat(60,60, Color.blue, true, 25);
		
		testListe.neueFigurHinzufuegen(kreis1);
		testListe.neueFigurHinzufuegen(kreis2);
		testListe.neueFigurHinzufuegen(quad1);
		testListe.neueFigurHinzufuegen(quad2);
		testListe.neueFigurHinzufuegen(recht1);
		testListe.neueFigurHinzufuegen(recht2);
	}


	// -----------------------------------------------------------------------------------
	@Test
	@Order(21)
	void testNeueFigur()
	{
		assertTrue(testListe.istFigurBereitsInDerListe(quad1), errReturnValue + "istFigurBereitsInDerListe");

		Rechteck neu = new Rechteck(90,90, Color.green, true, 45,25);
		assertFalse(testListe.istFigurBereitsInDerListe(neu), errReturnValue + "istFigurBereitsInDerListe");

		testListe.neueFigurHinzufuegen(neu);
		assertTrue(testListe.istFigurBereitsInDerListe(neu), errReturnValue + "istFigurBereitsInDerListe");
	}


	@Test
	@Order(22)
	void testFigurEntfernen()
	{
		assertTrue(testListe.istFigurBereitsInDerListe(recht2), errReturnValue + "istFigurBereitsInDerListe");
		
		testListe.entferneFigurAusDerListe(recht2);
		
		assertFalse(testListe.istFigurBereitsInDerListe(recht2), errReturnValue + "istFigurBereitsInDerListe");
	}

	@Test
	void testPrintListe()
	{
		assertDoesNotThrow(() -> {
			testListe.printFigurenListe();
			testListe.printKreisListe();
		}, errNoException );
	}

	@Test
	void testAlleMethodenTesten()
	{
		G2D.setVisible();
		assertDoesNotThrow(() -> {
			testListe.alleFigurenAufZufaelligePosition();
			testListe.alleFigurenZeichnen();
			testListe.alleKreiseAufZufaelligePosition();
			testListe.alleKreiseZeichnen();
			testListe.alleQuadrateZeichnen();
			testListe.alleRechteckeZeichnen();
			testListe.alleRechteckeZufaelligeFaerben();
		}, errNoException );
	}

	@Test
	void testGetAnzahlTesten()
	{
		assertEquals(2,testListe.getAnzKreis(),				 errReturnValue + "getAnzKreis");
		assertEquals(2,testListe.getAnzRechteck(),			 errReturnValue + "getAnzRechteck");
		assertEquals(2,testListe.getAnzQuadrat(),			 errReturnValue + "getAnzQuadrat");
		assertEquals(1,testListe.getAnzKreisGroesserAls(20), errReturnValue + "getAnzKreisGroesserAls");
		assertEquals(4,testListe.getAnzRechteckUndQuadrat(), errReturnValue + "getAnzRechteckUndQuadrat");
	}


	@Test
	void testGetKreise()
	{
		// ----------------------------------------------
		List<Figur> v = new ArrayList<>();
		v.add(kreis1);
		v.add(kreis2);
		assertEquals(v, testListe.getKreise(), errReturnListe + "getKReise");
	}
	@Test
	void testGetQuadrate()
	{
		// ----------------------------------------------
		List<Figur> v = new ArrayList<>();
		v.add(quad1);
		v.add(quad2);
		assertEquals(v, testListe.getQuadrate(), errReturnListe + "getQuadrate");
	}
	@Test
	void testGetRechtecke()
	{
		// ----------------------------------------------
		List<Figur> v = new ArrayList<>();
		v.add(recht1);
		v.add(recht2);
		assertEquals(v, testListe.getRechtecke(), errReturnListe + "getRechtecke");
	}

	@Test
	void testGetQuadrateGroesserAls()
	{
		// ----------------------------------------------
		List<Figur> v = new ArrayList<>();
//		v.add(quad1);   // Seitenlänge ist zu klein
		v.add(quad2);
		assertEquals(v, testListe.getQuadrateGroesserAls(20), errReturnListe + "getQuadrateGroesserAls");
	}

}
